import inspect
import textwrap


def get_base_shaper_tapply(columns, fun_name, output_signature):
    output_signature_str = get_base_shaper(output_signature)
    # print(output_signature)



    # output_signature_final = inspect.cleandoc(output_signature_str)
    code_string = """import nzae
                            import pandas as pd
                            class BaseShaperUdtf(nzae.Ae):
                                 def _runUdtf(self):
                                   
                                   df = pd.DataFrame(self.__iter__(), columns=""" + str(columns) + """ )
                                   value = self.""" + fun_name + """(df)
                                   #rows_list.append(str(self.getDatasliceId())+str(self.getHardwareId()))  
                                  

                                 def _runShaper(self):
                                    
                                    """ + textwrap.indent(output_signature_str, '                      ') + """
                                    




                                   """

    code_string = code_string.replace("'", "''")
    return inspect.cleandoc(code_string)

def get_base_shaper_groupedapply(columns, fun_name, output_signature):
    output_signature_str = get_base_shaper(output_signature)




        # output_signature_final = inspect.cleandoc(output_signature_str)
    code_string = """import nzae
                        import pandas as pd
                        class BaseShaperUdtf(nzae.Ae):
                             def _runUdtf(self):
                               rows_list=[]
                               for row in self:
                                  row_number = row[0]
                                  count = row[1]

                                  if row_number==1:
                                    rows_list=[]
                                  rows_list.append(row[2:])
                                  #rows_list.append(str(self.getDatasliceId())+str(self.getHardwareId()))  
                                  if row_number==count:                                                              
                                   df = pd.DataFrame(rows_list, columns=""" + str(columns) + """ )
                                   value = self.""" + fun_name + """(df)
                                   #self.output(value)
                                   #self.output(len(rows_list))




                             def _runShaper(self):
                               
                                     """ + textwrap.indent(output_signature_str, '                      ') + """



                               """

    code_string = code_string.replace("'", "''")
    return inspect.cleandoc(code_string)

def get_base_shaper(output_signature):
    output_signature_str = ""
    # print(output_signature)

    for column in output_signature:
        # print(column)
        if output_signature[column] == 'int' :
            column_val = 'self.DATA_TYPE__INT32'

            output_signature_str += """
                                                       self.addOutputColumn('""" + column + """',""" + column_val + """) """
        if output_signature[column] == 'int64':
            column_val = 'self.DATA_TYPE__INT64'

            output_signature_str += """
                                                       self.addOutputColumn('""" + column + """',""" + column_val + """) """

        if output_signature[column] == 'bool' :
            column_val = 'self.DATA_TYPE__INT32'
            output_signature_str += """
                                                       self.addOutputColumn('""" + column + """',""" + column_val + """) """

        if output_signature[column] == 'float' or output_signature[column] == 'float64':
            column_val = 'self.DATA_TYPE__FLOAT'
            output_signature_str += """
         
                                                       self.addOutputColumn('""" + column + """',""" + column_val + """) """


        if output_signature[column] == 'double':
            column_val = 'self.DATA_TYPE__DOUBLE'
            output_signature_str += """
                                                       self.addOutputColumn('""" + column + """',""" + column_val + """) """
        if output_signature[column] == 'str' or output_signature[column] == 'object':
            column_val = 'self.DATA_TYPE__VARIABLE'
            output_signature_str += """
                                                       self.addOutputColumnString('""" + column + """',""" + column_val + """,1000) """
    return output_signature_str


def get_base_shaper_apply(columns, fun_name, output_signature):

    output_signature_str = get_base_shaper(output_signature)
    # output_signature_final = inspect.cleandoc(output_signature_str)
    code_string = """import nzae
                            import pandas as pd
                            class BaseShaperUdtf(nzae.Ae):
                                 def _runUdtf(self):
                                  
                                   for row in self:
                                     row = self.apply_fun(row)  
                                   

                                 def _runShaper(self):
                                   """ + textwrap.indent(output_signature_str, '                      ') + """




                                   """

    code_string = code_string.replace("'", "''")
    return inspect.cleandoc(code_string)


def get_base_shaper_install(output_signature, package_name):
    output_signature_str = get_base_shaper(output_signature)
    # output_signature_final = inspect.cleandoc(output_signature_str)
    code_string = """import nzae
                            
                            class BaseShaperUdtf(nzae.Ae):
                                 def _runUdtf(self):

                                    import subprocess
                                    cwd = os.getcwd()
                                    
                                    for row in self:
                                                                
                                        
                                        res = subprocess.call([\'sh', \'/nz/export/ae/adapters/python3x/11/lib/installPackage.sh',  """+"\'"+package_name+"\'"+"""])
                                       
                                        self.output(res)
                                       
                                    


                                 def _runShaper(self):
                                   """ + textwrap.indent(output_signature_str, '       ') + """




                                   """

    code_string = code_string.replace("'", "''")
    return inspect.cleandoc(code_string)











#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Configuration for pytest testing framework module
=================================================

To launch the test routine, do in ibmdbpy-master folder: py.test
Per default, py.test routine assumes a data source "BLUDB" is defined in ODBC
settings, with userID and password, and test with the well-known 'iris' dataset.
This can be changed using command line options, defined as following :

Options
-------
dsn
    Data source name to use for testing

uid
    UserID credential to connect to the data source

pwd
    Password credential to connect to the data source

hostname
    Name of the host to connect to with nzpy

jdbc
    JDBC url string to be used to connect to the data source, should comply
    to this template : jdbc:db2://<HOST>:<PORT>/<DBNAME>:user=<UID>;password=<PWD>

table
    Dataset to use for testing

Examples
--------
py.test
    Launch py.test routine using iris sampledataset in "BLUDB" ODBC data source.

py.test --dsn=<DSN>
    Do the test routine with the data source <DSN> as defined in ODBC settings.

py.test --dsn=<DSN> --uid=<UID> --pwd=<pwd>
    In case userID and password are not stored in ODBC settings.

py.test --dsn=<DSN> --uid=<UID> --pwd=<pwd> --hostname <hostname>
    In case to connect to database with the name <DSN> on host <hostname>
    with uid <UID> and password <password>

py.test --jdbc=<jdbc_url_string>
    Do the test routine using jdbc connection

py.test --table=<DATA>
    Do the test routine using the dataset <DATA>


Notes
-----
Several sample datasets are provided for testing : titanic, iris, swiss
All options can be combined, however:
    * 'uid' and 'pwd' cannot be defined if 'jdbc' is defined.
    * 'uid' and 'password' must both be defined if one of them is already given.
    * for nzpy connections dsn, uid, pwd and hostname must be defined

"""

# Python 2 compatibility
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import str
from future import standard_library
standard_library.install_aliases()

import pytest
import nzpyida

def pytest_addoption(parser):
    """
    Definition of admissible options for the pytest command line
    """
    parser.addoption("--table", default="iris",
        help="Name of the table to test the dataset")
    parser.addoption("--dsn", default="BLUDB",
        help="Data Source Name")
    parser.addoption("--uid", default='',
        help="User ID")
    parser.addoption("--pwd", default='',
        help="Password")
    parser.addoption("--jdbc", default='',
        help="jdbc url string for JDBC connection")
    parser.addoption("--hostname", default='',
        help="hostname for nzpy connection")

def get_data(request):
    """
    Helper function that returns the dataset correponding to the table option
    (default, iris). Raise an error in case the dataset is not included in
    ibmdbpy or its name is unkown.
    """
    table = request.config.getoption('--table')
    if table == "titanic":
        from nzpyida.sampledata.titanic import titanic
        return titanic
    elif table == "iris":
        from nzpyida.sampledata.iris import iris
        return iris
    elif table == "swiss":
        from nzpyida.sampledata.swiss import swiss
        return swiss
    else:
        raise ValueError("Unknown table " + str(table))

# Fixture definitions

@pytest.fixture(scope="session")
def idadb(request):
    """
    DataBase connection fixture, to be used for the whole testing session.
    Hold the main IdaDataBase object. Shall not be closed except by a
    pytest finalizer.
    """
    def fin():
        try:
            idadb.close()
        except:
            pass
    request.addfinalizer(fin)

    hostname = request.config.getoption('--hostname')
    jdbc = request.config.getoption('--jdbc')

    if hostname != '':
        try:
            idadb = nzpyida.IdaDataBase(
                dsn={'database': request.config.getoption('--dsn'), 'host': hostname, 'port': 5480},
                uid=request.config.getoption('--uid'),
                pwd=request.config.getoption('--pwd'),
                autocommit=False)
        except:
            raise
    elif jdbc != '':
        try:
            idadb = nzpyida.IdaDataBase(dsn=jdbc, autocommit=False)
        except:
            raise
    else:
        try:
            idadb = nzpyida.IdaDataBase(dsn=request.config.getoption('--dsn'),
                                        uid=request.config.getoption('--uid'),
                                        pwd=request.config.getoption('--pwd'),
                                        autocommit=False)
        except:
            raise
    return idadb

@pytest.fixture(scope="function")
def idadb_tmp(request):
    """
    DataBase connection fixture, to be used by destructive and
    semi-destructive functions.
    """
    def fin():
        try:
            idadb_tmp.close()
        except:
            pass
    request.addfinalizer(fin)

    jdbc = request.config.getoption('--jdbc')
    hostname = request.config.getoption('--hostname')
    if hostname != '':
        try:
            idadb_tmp = nzpyida.IdaDataBase(
                dsn={'database': request.config.getoption('--dsn'), 'host': hostname, 'port': 5480},
                uid=request.config.getoption('--uid'),
                pwd=request.config.getoption('--pwd'),
                autocommit=False)

        except:
            raise
    elif jdbc != '':
        try:
            idadb_tmp = nzpyida.IdaDataBase(dsn=jdbc, autocommit=False)
        except:
            raise
    else:
        try:
            idadb_tmp = nzpyida.IdaDataBase(dsn=request.config.getoption('--dsn'),
                                            uid=request.config.getoption('--uid'),
                                            pwd=request.config.getoption('--pwd'),
                                            autocommit=False)
        except:
            raise
    return idadb_tmp

@pytest.fixture(scope="session")
def idadf(request, idadb):
    """
    IdaDataFrame fixture to be used for the whole testing session.
    Shall not be used during the testing procedure by destructive and
    semi-destructive functions.
    """
    data = get_data(request)

    def fin():
        try:
            idadb.drop_table("TEST_IBMDBPY")
            idadb.commit()
        except:
            pass
    request.addfinalizer(fin)

    idadf = idadb.as_idadataframe(data, 'TEST_IBMDBPY', clear_existing = True)
    idadb.commit()
    return idadf

@pytest.fixture(scope="session")
def idadf_onecolumn_numeric(idadf):
    """

    IdaDataFrame fixture with a single numeric column to be used for the whole
    testing session.
    Shall not be used during the testing procedure by destructive and
    semi-destructive functions.
    """
    table_def = idadf._table_def()
    numeric_column = table_def[table_def["VALTYPE"] == "NUMERIC"].index.tolist()[:1]
    single_column_dataframe = idadf[numeric_column]
    return single_column_dataframe

@pytest.fixture(scope="session")
def idadf_indexer(request, idadb, idadf):
    """
    IdaDataFrame fixture with indexer defined to be used for the whole testing session.
    Shall not be used during the testing procedure by destructive and
    semi-destructive functions.
    """
    data = get_data(request)

    def fin():
        try:
            idadb.drop_table("TEST_IBMDBPY_INDEX")
            idadb.commit()
        except:
            pass
    request.addfinalizer(fin)

    idadf_index = idadb.as_idadataframe(data, 'TEST_IBMDBPY_INDEX',
                                        clear_existing=True,
                                        indexer=idadf.columns[len(idadf.columns) - 1])

    idadb.commit()
    return idadf_index

@pytest.fixture(scope="function")
def idadf_tmp(request, idadb):
    """
    IdaDataFrame fixture to be used by destructive and semi-destructive
    functions. To be considered as a temporary DataFrame that is created
    and destroyed for each function that requires it.
    """
    data = get_data(request)

    def fin():
        try:
            idadb.drop_table("TEST_IBMDBPY_TMP")
            idadb.commit()
        except:
            pass
    request.addfinalizer(fin)

    idadf = idadb.as_idadataframe(data, 'TEST_IBMDBPY_TMP', clear_existing = True)
    idadb.commit()
    return idadf

@pytest.fixture(scope="function")
def idageodf_county(idadb):
    """
    IdaGeoDataFrame to test geospatial methods
    This refers to the table 'GEO_COUNTY' of the 'SAMPLES' schema
    The table has one geometry column named 'SHAPE' of type 'ST_MULTIPOLYGON'
    Don't use it for destructive nor non-destructive methods (modify columns)
    """
    idageodf = nzpyida.IdaGeoDataFrame(idadb, 'SAMPLES.GEO_COUNTY', indexer='OBJECTID')
    return idageodf

@pytest.fixture(scope="function")
def idageodf_customer(idadb):
    """
    IdaGeoDataFrame to test geospatial methods
    This refers to the table 'GEO_CUSTOMER' of the 'SAMPLES' schema
    The table has one geometry column named 'SHAPE' of type 'ST_POINT'
    Don't use it for destructive nor non-destructive methods (modify columns)
    """
    idageodf = nzpyida.IdaGeoDataFrame(idadb, 'SAMPLES.GEO_CUSTOMER', indexer='OBJECTID')
    return idageodf

@pytest.fixture(scope="function")
def idageodf_tornado(idadb):
    """
    IdaGeoDataFrame to test geospatial methods
    This refers to the table 'GEO_TORNADO' of the 'SAMPLES' schema
    The table has one geometry column named 'SHAPE' of type 'ST_MULTILINESTRING'
    Don't use it for destructive nor non-destructive methods (modify columns)
    """
    idageodf = nzpyida.IdaGeoDataFrame(idadb, 'SAMPLES.GEO_TORNADO')
    return idageodf

@pytest.fixture(scope="session")
def idaview(request, idadb, idadf):
    """
    IdaDataFrame fixture to be used for the whole testing session. Open a view
    based on idadf fixture.
    """
    def fin():
        try:
            idadb.drop_view("TEST_VIEW_ibmdbpy")
            idadb.commit()
        except:
            pass
    request.addfinalizer(fin)

    if idadb.exists_view("TEST_VIEW_ibmdbpy"):
        idadb.drop_view("TEST_VIEW_ibmdbpy")

    idadb._create_view(idadf, "TEST_VIEW_ibmdbpy")
    return nzpyida.IdaDataFrame(idadb, "TEST_VIEW_ibmdbpy")

@pytest.fixture(scope="function")
def idaview_tmp(request, idadb, idadf):
    """
    IdaDataFrame fixture to be used by destructive and semi-destructive
    functions. To be considered as a temporary DataFrame that is created
    and destroyed for each function that requires it. Opens a view based on
    idadf fixture.
    """
    def fin():
        try:
            idadb.drop_view("TEST_VIEW_ibmdbpy_TMP")
            idadb.commit()
        except:
            pass
    request.addfinalizer(fin)

    if idadb.exists_view("TEST_VIEW_ibmdbpy_TMP"):
        idadb.drop_view("TEST_VIEW_ibmdbpy_TMP")

    idadb._create_view(idadf, "TEST_VIEW_ibmdbpy_TMP")
    return nzpyida.IdaDataFrame(idadb, "TEST_VIEW_ibmdbpy_TMP")

@pytest.fixture(scope="function")
def idageodf_county_view(request, idadb):
    """
    IdaGeoDataFrame to test geospatial methods with lowercase column names.
    It refers to the view 'GEO_COUNTY_VIEW' where the columns of the
    'SAMPLES.GEO_COUNTY' table have been renamed to their lowercase counterparts.
    The view has one geometry column named 'shape' of type 'ST_MULTIPOLYGON'.
    Don't use it for destructive nor non-destructive methods (modify columns).
    """
    def fin():
        try:
            idadb.drop_view("GEO_COUNTY_VIEW")
            idadb.commit()
        except:
             pass
    request.addfinalizer(fin)
    idadb.ida_query('CREATE OR REPLACE VIEW GEO_COUNTY_VIEW AS ' +
        '(SELECT "OBJECTID" as "objectid", "NAME" as "name", "SHAPE" as "shape"  FROM SAMPLES.GEO_COUNTY)')
    idageodf = nzpyida.IdaGeoDataFrame(idadb, 'GEO_COUNTY_VIEW', indexer='objectid')
    return idageodf

@pytest.fixture(scope="session")
def df(request):
    """
    Original physical dataset to be used to testing. To be used by
    non-destructive function on the whole testing procedure.
    """
    return get_data(request)

@pytest.fixture(scope = "session", autouse=True)
def session_setup(idadb, request):
    """
    Defines actions to be done before starting the testing procedure.
    """
    # empty
    return

@pytest.fixture(scope = "session", autouse=True)
def session_teardown(idadb, idadf, idaview, request):
    """
    Defines cleanup actions to be done once the testing procedure is done.
    """
    def fin():
        try:
            idadb.drop_table(idadf.name)
            idadb.drop_view(idaview.name)
            idadb.commit()
            idadb.close()
        except: pass
    request.addfinalizer(fin)
    return

#def pytest_generate_tests(metafunc):
#    if 'table' in metafunc.fixturenames:
#        metafunc.parametrize("table",
#                             metafunc.config.option.table, scope = 'session')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
# -----------------------------------------------------------------------------


from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
from .tapply import NZFunTApply
from .apply import NZFunApply
from .groupedapply import NZFunGroupedApply
from .tapply_class import NZClassTApply
from .install import NZInstall

standard_library.install_aliases()

__all__ = ['NZFunTApply', 'NZClassTApply', 'NZFunApply', 'NZFunGroupedApply', 'NZInstall']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import pytest

from nzpyida import IdaDataFrame
from nzpyida.learn import KMeans

class Test_DeleteDataBaseObjects(object):

    def test_idadb_drop_table(self, idadb, idadf_tmp):
        assert(idadb.exists_table(idadf_tmp.name) == 1)
        idadb.drop_table(idadf_tmp.name)
        assert(idadb.exists_table(idadf_tmp.name) == 0)

    def test_idadb_drop_table_value_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.drop_table("NOTEXISTINGOBJECT_496070383095079384063739509")

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_drop_table_type_error(self, idadb, idaview):
        with pytest.raises(TypeError):
            idadb.drop_table(idaview.name) # this is a view

    def test_idadb_drop_view(self, idadb, idaview_tmp):
        assert(idadb.exists_view(idaview_tmp.name) == 1)
        idadb.drop_view(idaview_tmp.name)
        assert(idadb.exists_view(idaview_tmp.name) == 0)

    def test_idadb_drop_view_value_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.drop_view("NOTEXISTINGOBJECT_496070383095079384063739509")

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_drop_view_type_error(self, idadb, idadf):
        with pytest.raises(TypeError):
            idadb.drop_view(idadf.name) # this is a table

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    @pytest.mark.xfail(raises=ValueError)
    def test_idadb_drop_model_positive(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, destructive = True)
        # Create a simple KMEANS model
        kmeans = KMeans(n_clusters = 3)
        kmeans.fit(idadf_tmp)
        assert(idadb.is_model(kmeans.modelname) == 1)
        idadb.drop_model(kmeans.modelname)
        idadb.commit()

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_drop_model_value_error(self, idadb):
        with pytest.raises(ValueError):
            idadb.is_model("NOTEXISTINGOBJECT_496070383095079384063739509")

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc') or config.getvalue('hostname') != ''")
    def test_idadb_drop_model_type_error(self, idadb, idadf, idaview):
        with pytest.raises(TypeError):
            idadb.drop_model(idadf.name)
        with pytest.raises(TypeError):
            idadb.drop_model(idaview.name)
        #with pytest.raises(TypeError):
        #    idadb.drop_model("ST_INFORMTN_SCHEMA.ST_UNITS_OF_MEASURE")

class Test_TableManipulation(object):
    def test_idadb_rename(self, idadb, idadf_tmp):
        try:
            idadb.drop_table("TEST_RENAMED")
        except:
            pass
        original_name = idadf_tmp.tablename
        idadb.rename(idadf_tmp, "TEST_RENAMED")
        idadb.commit()
        assert(idadf_tmp.name == "TEST_RENAMED")
        assert(idadb.exists_table("TEST_RENAMED") == 1)
        assert(idadb.exists_table(original_name) == 0)
        idadb.rename(idadf_tmp, original_name)

    def test_idadb_rename_value_error(self, idadb, idadf):
        with pytest.raises(ValueError):
            idadb.rename(idadf, "T569ß4359**4\4%")

    @pytest.mark.skipif("'netezza' in config.getvalue('jdbc')")
    def test_idadb_rename_type_error(self, idadb, idaview):
        with pytest.raises(TypeError):
            idadb.rename(idaview, "TEST_VIEW_RENAME")

    def test_idadb_rename_name_error(self, idadb, idadf_tmp):
        with pytest.raises(NameError):
            idadb.rename(idadf_tmp, idadf_tmp.tablename)

    def test_idadb_add_column_id_destructive(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "ID", destructive = True)
        assert("ID" in idadf_tmp.columns)
        assert("ID" == idadf_tmp.indexer)
        assert("ID" in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name, indexer = "ID")
        assert("ID" in idadf_tmp_new.columns)
        assert("ID" == idadf_tmp_new.indexer)
        assert("ID" in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_add_column_id_non_destructive(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "ID", destructive = False)
        assert("ID" in idadf_tmp.columns)
        assert("ID" == idadf_tmp.indexer)
        assert("ID" not in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert("ID" not in idadf_tmp_new.columns)
        assert("ID" not in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_add_column_id_non_destructive_custom_name(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "MY_CUSTOM_ID", destructive = False)
        assert("MY_CUSTOM_ID" in idadf_tmp.columns)
        assert("MY_CUSTOM_ID" == idadf_tmp.indexer)
        assert("MY_CUSTOM_ID" not in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert("MY_CUSTOM_ID" not in idadf_tmp_new.columns)
        assert("MY_CUSTOM_ID" not in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_add_column_id_non_destructive_value_error(self, idadb, idadf_tmp):
        with pytest.raises(ValueError):
            idadb.add_column_id(idadf_tmp, idadf_tmp.columns[0], destructive = False)

    def test_idadb_add_column_id_non_destructive_type_error(self, idadb, df):
        with pytest.raises(TypeError):
            idadb.add_column_id(df, "ID", destructive = False)

    def test_idadb_delete_column_value_error(self, idadb, idadf):
        with pytest.raises(ValueError):
            idadb.delete_column(idadf, "NOTEXISTING_COLUMN_309599439")

    def test_idadb_delete_column_type_error(self, idadb, idadf):
        with pytest.raises(TypeError):
            idadb.delete_column(idadf, idadf)

    def test_idadb_delete_column_destructive(self, idadb, idadf_tmp):
        to_delete = idadf_tmp.columns[0]
        idadb.delete_column(idadf_tmp, to_delete, destructive = True)
        assert(to_delete not in idadf_tmp.columns)
        assert(to_delete not in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert(to_delete not in idadf_tmp_new.columns)
        assert(to_delete not in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_delete_column_destructive_real_indexer(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "ID", destructive = True)
        idadb.delete_column(idadf_tmp, "ID", destructive = True)
        assert("ID" not in idadf_tmp.columns)
        assert("ID" != idadf_tmp.indexer)
        assert("ID" not in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert("ID" not in idadf_tmp_new.columns)
        assert("ID" not in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_delete_column_destructive_virtual_indexer(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "ID", destructive = False)
        idadb.delete_column(idadf_tmp, "ID", destructive = True)
        assert("ID" not in idadf_tmp.columns)
        assert("ID" != idadf_tmp.indexer)
        assert("ID" not in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert("ID" not in idadf_tmp_new.columns)
        assert("ID" not in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_delete_column_non_destructive(self, idadb, idadf_tmp):
        to_delete = idadf_tmp.columns[0]
        idadb.delete_column(idadf_tmp, to_delete, destructive = False)
        assert(to_delete not in idadf_tmp.columns)
        assert(to_delete in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert(to_delete in idadf_tmp_new.columns)
        assert(to_delete in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_delete_column_non_destructive_real_indexer(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "ID", destructive = True)
        idadb.delete_column(idadf_tmp, "ID", destructive = False)
        assert("ID" not in idadf_tmp.columns)
        assert("ID" != idadf_tmp.indexer)
        assert("ID" in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name, indexer = "ID")
        assert("ID" in idadf_tmp_new.columns)
        assert("ID" == idadf_tmp_new.indexer)
        assert("ID" in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_delete_column_non_destructive_virtual_indexer(self, idadb, idadf_tmp):
        idadb.add_column_id(idadf_tmp, "ID", destructive = False)
        idadb.delete_column(idadf_tmp, "ID", destructive = False)
        assert("ID" not in idadf_tmp.columns)
        assert("ID" != idadf_tmp.indexer)
        assert("ID" not in idadf_tmp._get_all_columns_in_table())
        idadf_tmp_new = IdaDataFrame(idadb, idadf_tmp._name)
        assert("ID" not in idadf_tmp_new.columns)
        assert("ID" not in idadf_tmp_new._get_all_columns_in_table())

    def test_idadb_append(self, idadb, idadf_tmp, df):
        nrow = idadf_tmp.shape[0]
        ncol = idadf_tmp.shape[1]
        idadb.append(idadf_tmp, df)
        assert(idadf_tmp.shape[0] == nrow*2)
        nrow = idadf_tmp.shape[0]
        idadb.append(idadf_tmp, df.loc[0:4])
        assert(idadf_tmp.shape[0] == nrow + 5)
        assert(idadf_tmp.shape[1] == ncol)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Classes used for subsetting IdaDataFrames.
Indexer currently available : loc
"""
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

import warnings

import six
from nzpyida.exceptions import IdaDataBaseError

#----------------------------------------------------------------------------
# Class for the loc object for IdaDataFrames

class Loc(object):
        """
        The Loc class is used to select and project IdaDataFrames.
        It implements a Pandas-like interface.
        """
        def __init__(self, idadf):
            self.idadf = idadf

        def __getitem__(self, key):
            """
            Use the loc object of an IdaDataFrame or IdaSeries instance to
            do projection or selection in a table.
            
            Notes
            -----
            The determinism of the result is guaranteed only if the IdaDataFrame
            has a valid indexer. 
            
            Examples
            --------
            >>> idairis.loc[0:49] # Select the first 50 rows
            >>> idairis.loc[2, "SepalLength"] # Select the 3rd rows and column "SepalLength"
            >>> idairis.loc[0:len(idairis), ["SepalLength", "SepalWidth"]] # Select all rows and columns "SepalLength", "SepalWidth"
            """
            if type(key) is tuple:
                if len(key) > 2:
                    raise ValueError("Too many indexer (expected 2)")
                else:
                    index = key[0]
                    cols = key[1]
            else:
                index = key
                cols = None


            if isinstance(index, six.integer_types):
                if self.idadf.indexer is None:
                    if (index < 0)|(index > self.idadf.shape[0]):
                        raise KeyError("The label [%s] is not in the [index]" %(index))
            if isinstance(index, list):
                if self.idadf.indexer is None:
                    if False in [isinstance(x, six.integer_types) for x in index]:
                        raise IdaDataBaseError("The IdaDataFrame has no indexer, so 'index' should be an integer or a list of integers")
                    for x in index:
                        if (x < 0)|(x > self.idadf.shape[0]):
                            raise ValueError("The index [%s] is out of range" %(index))

            if cols is not None:
                if isinstance(cols, six.string_types):
                    if cols not in self.idadf.columns:
                            raise KeyError("The label %s is not in the [columns]" %(cols))
                    newidadf = self.idadf._clone_as_serie(cols)
                else:
                    not_existing = [col for col in cols if col not in self.idadf.columns]
                    if not_existing:
                        raise KeyError("The label [%s] is not in the [columns]" %(not_existing))
                    newidadf = self.idadf._clone()

                for col in newidadf.internal_state.columndict.keys():
                    if col not in cols:
                        del newidadf.internal_state.columndict[col]

                if self.idadf.indexer is not None:
                    if self.idadf.indexer not in cols:
                        newidadf.internal_state.columndict[self.idadf.indexer] = "\""+self.idadf.indexer+"\""

                newidadf.internal_state.index = index
                newidadf.internal_state.update()
                #newidadf._reset_attributes(["shape", "dtypes", "index", "columns"]) # this was causing troubles 

                if self.idadf.indexer is not None:
                    if self.idadf.indexer not in cols:
                        
                        del newidadf.internal_state.columndict[self.idadf.indexer]
                        newidadf.internal_state.update()
                        newidadf._reset_attributes(["shape", "dtypes", "index", "columns"])


            else:
                newidadf = self.idadf._clone()
                newidadf.internal_state.index = index
                newidadf.internal_state.update()
                newidadf._reset_attributes(["shape", "index"])

            if self.idadf.indexer is None:
                if not " ORDER BY " in self.idadf.internal_state.get_state():
                    warnings.warn("Row order is not guaranteed if no indexer" +
                                  " was given and the dataset was not sorted")
            return newidadf


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()

class Test_Filtering(object):
    # Test combinations
    def test_Filtering_lt(self, idadf):
        mean = idadf.mean()
        if not mean.empty:
            ida = idadf[idadf[mean.index[0]] < mean[0]]
            assert(ida.max()[0] < mean[0])
            
    def test_Filtering_le(self, idadf):
        mean = idadf.mean()
        if not mean.empty:
            ida = idadf[idadf[mean.index[0]] <= mean[0]]
            assert(ida.max()[0] <= mean[0])
        pass

    def test_Filtering_eq(self, idadf):
        maxi = idadf.max()
        if not maxi.empty:
            ida = idadf[idadf[maxi.index[0]] == maxi[0]]
            assert(ida.max()[0] == maxi[0])
            assert(ida.min()[0] == maxi[0])
        pass
    
    def test_Filtering_neq(self, idadf):
        maxi = idadf.max()
        if not maxi.empty:
            ida = idadf[idadf[maxi.index[0]] != maxi[0]]
            assert(ida.max()[0] != maxi[0])
            assert(ida.min()[0] != maxi[0])
        pass

    def test_Filtering_ge(self, idadf):
        mean = idadf.mean()
        if not mean.empty:
            ida = idadf[idadf[mean.index[0]] >= mean[0]]
            assert(ida.max()[0] >= mean[0])
        pass

    def test_Filtering_gt(self, idadf):
        mean = idadf.mean()
        if not mean.empty:
            ida = idadf[idadf[mean.index[0]] > mean[0]]
            assert(ida.max()[0] > mean[0])
        pass


class Test_FilterQuery(object):
    # Test combinations
    def test_FilterQuery_and(self, idadf):
        maxi = idadf.max()
        mini = idadf.min()
        if not (maxi.empty | mini.empty):
            ida = idadf[(idadf[mini.index[0]] > mini[0])&(idadf[maxi.index[0]] < maxi[0])]
            assert(ida.max()[0] < maxi[0])
            assert(ida.min()[0] > mini[0])
        pass

    def test_FilterQuery_or(self, idadf):
        maxi = idadf.max()
        mini = idadf.min()
        if not (maxi.empty | mini.empty):
            ida = idadf[(idadf[mini.index[0]] == mini[0])|(idadf[maxi.index[0]] == maxi[0])]
            head = ida.head()
            for value in head.values:
                assert((value[0] == mini[0])|(value[0] == maxi[0]))         
        pass

    def test_FilterQuery_xor(self, idadf):
        mini = idadf.min()
        if not mini.empty:
            ida = idadf[(idadf[mini.index[0]] >= mini[0])^(idadf[mini.index[0]] == mini[0])]
            assert(ida.min()[0] > mini[0])
        pass

    def test_FilterQuery_error(self, idadf):
        pass